#!/usr/bin/env python
# -*- coding: utf-8 -*-

# author: Pocholo
# date: 8/12/22

# Comments directed to http://gimpchat.com

# Installation:
# This script should be placed in the user plugins folder

# Copyright (C)

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by   
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.

from gimpfu import *
import os, sys,time
import gettext
gettext.install("gimp20", gimp.locale_directory, unicode=True)

def pm_candy_cane_text(customtext, fontname, offset_x, offset_y, blurRadius, opacity, stripcolor_1, stripcolor_2, chrCane):

    #Initiates the temporary state
    pdb.gimp_context_push()
    pdb.gimp_context_set_defaults()
    
    size = 250
    img = pdb.gimp_image_new(1, 1, RGB)
    pdb.gimp_context_set_foreground("#b1b1b1")
    textLayer = pdb.gimp_text_fontname(img, None, 0, 0, customtext, 100, 1, size, PIXELS, fontname)
    pdb.gimp_text_layer_set_justification(textLayer, 2)
    #Resize text
    img.resize(textLayer.width, textLayer.height, 0, 0)
    #Rename the text layer and move up
    pdb.gimp_item_set_name(textLayer, "Text")
    textLayer = pdb.gimp_image_get_layer_by_name(img, "Text")
    x = textLayer.offsets[0]
    y = textLayer.offsets[1]
    x = (img.width - textLayer.width) / 2
    y = (img.height - textLayer.height) / 2
    textLayer.set_offsets(x, y)
    pdb.gimp_layer_resize_to_image_size(textLayer)
    
    pdb.gimp_display_new(img)
    
    #Create background
    backLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Background", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_insert_layer(img, backLayer, None, 1)
    pdb.gimp_context_set_background("#73bde0")
    pdb.gimp_drawable_fill(backLayer, FILL_BACKGROUND)
    
    pdb.script_fu_drop_shadow(img, textLayer, offset_x, offset_y, blurRadius, "Black", opacity, FALSE)
    
    #Bump the text 
    pdb.gimp_context_set_default_colors()
    bumpLayer = pdb.gimp_layer_new_from_drawable(textLayer, img)
    pdb.gimp_image_insert_layer(img, bumpLayer, None, 0)
    pdb.gimp_item_set_name(bumpLayer, "Bumpmap")
    pdb.gimp_layer_set_mode(bumpLayer, LAYER_MODE_NORMAL)
    pdb.gimp_layer_set_opacity(bumpLayer, 100)
    pdb.gimp_layer_flatten(bumpLayer)
    pdb.plug_in_gauss(img, bumpLayer, 40, 40, 1)   #method=1
    pdb.gimp_layer_set_opacity(bumpLayer, 0)
    pdb.plug_in_bump_map(img, textLayer, 
                                bumpLayer,   #bumpmap
                                      270,   #azimuth
                                       45,   #elevation
                                       55,   #depth
                                        0,   #xofs
                                        8,   #yofs
                                        0,   #waterlevel
                                        0,   #ambient
                                     TRUE,   #compensate
                                    FALSE,   #invert
                                        0)   #type - SPHERICAL
    
    #Smooth the text
    pdb.gimp_layer_set_lock_alpha(textLayer, TRUE)
    pdb.plug_in_gauss(img, textLayer, 5, 5, 1)   #method=1
    pdb.gimp_layer_set_lock_alpha(textLayer, FALSE)
    
    #Create the candy stripes pattern
    size = 250
    stripesPat = pdb.gimp_layer_new(img, size, size, RGB_IMAGE, "Candy Stripes", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_insert_layer(img, stripesPat, None, 1)
    pdb.gimp_drawable_fill(stripesPat, FILL_WHITE)
    pdb.gimp_brushes_set_brush("2. Block 01")
    pdb.gimp_context_set_brush_size(440)
    pdb.gimp_context_set_brush_aspect_ratio(17)
    pdb.gimp_context_set_brush_angle(-45)
    pdb.gimp_context_set_brush_default_hardness()
    pdb.gimp_context_set_brush_spacing(0.01)
    pdb.gimp_context_set_brush_force(1)
    #Color 1
    pdb.gimp_context_set_foreground(stripcolor_1)
    pdb.gimp_context_set_dynamics("Dynamics Off")  
    pdb.gimp_paintbrush_default(stripesPat, 2, [0, 0])
    pdb.gimp_paintbrush_default(stripesPat, 2, [65, 60])
    pdb.gimp_paintbrush_default(stripesPat, 2, [125, 125])
    pdb.gimp_paintbrush_default(stripesPat, 2, [187, 188])
    pdb.gimp_paintbrush_default(stripesPat, 2, [250, 250])
    #Color 2
    pdb.gimp_context_set_brush_size(440)
    pdb.gimp_context_set_brush_aspect_ratio(17)
    pdb.gimp_context_set_foreground(stripcolor_2)
    pdb.gimp_context_set_dynamics("Dynamics Off")  
    pdb.gimp_paintbrush_default(stripesPat, 2, [32, 30])
    pdb.gimp_paintbrush_default(stripesPat, 2, [127, 60])
    pdb.gimp_paintbrush_default(stripesPat, 2, [187, 125])
    pdb.gimp_paintbrush_default(stripesPat, 2, [219, 219])
    
    #Copy the pattern to clipboard and delete it
    non_empty = pdb.gimp_edit_copy(stripesPat)
    pdb.gimp_image_remove_layer(img, stripesPat)
    
    if chrCane==1:
        #Create the candy stripes pattern
        size = 250
        stripesPat = pdb.gimp_layer_new(img, size, size, RGB_IMAGE, "Candy Stripes", 100, LAYER_MODE_NORMAL)
        pdb.gimp_image_insert_layer(img, stripesPat, None, 1)
        pdb.gimp_drawable_fill(stripesPat, FILL_WHITE)
        pdb.gimp_brushes_set_brush("2. Block 01")
        pdb.gimp_context_set_brush_size(440)
        pdb.gimp_context_set_brush_aspect_ratio(17)
        pdb.gimp_context_set_brush_angle(-45)
        pdb.gimp_context_set_brush_default_hardness()
        pdb.gimp_context_set_brush_spacing(0.01)
        pdb.gimp_context_set_brush_force(1)
        #Color 1
        pdb.gimp_context_set_foreground(stripcolor_1)
        pdb.gimp_context_set_dynamics("Dynamics Off")  
        pdb.gimp_paintbrush_default(stripesPat, 2, [0, 0])
        pdb.gimp_paintbrush_default(stripesPat, 2, [65, 60])
        pdb.gimp_paintbrush_default(stripesPat, 2, [125, 125])
        pdb.gimp_paintbrush_default(stripesPat, 2, [187, 188])
        pdb.gimp_paintbrush_default(stripesPat, 2, [250, 250])
        #Color 2
        pdb.gimp_context_set_brush_size(440)
        pdb.gimp_context_set_brush_aspect_ratio(20)
        pdb.gimp_context_set_foreground(stripcolor_2)
        pdb.gimp_context_set_dynamics("Dynamics Off")  
        pdb.gimp_paintbrush_default(stripesPat, 2, [32, 30])
        pdb.gimp_paintbrush_default(stripesPat, 2, [127, 60])
        pdb.gimp_paintbrush_default(stripesPat, 2, [187, 125])
        pdb.gimp_paintbrush_default(stripesPat, 2, [219, 219])
        
        pdb.gimp_brushes_set_brush("2. Block 01")
        pdb.gimp_context_set_brush_size(5)
        pdb.gimp_context_set_brush_aspect_ratio(0)
        pdb.gimp_context_set_brush_angle(0)
        pdb.gimp_context_set_brush_default_hardness()
        pdb.gimp_context_set_brush_spacing(0.01)
        pdb.gimp_context_set_brush_force(1)
        pdb.gimp_context_set_dynamics("Dynamics Off")  
        pdb.gimp_eraser_default(stripesPat, 4, [0, 56.5, 56.5, 0])
        pdb.gimp_eraser_default(stripesPat, 4, [0, 69, 69, 0])
        pdb.gimp_eraser_default(stripesPat, 4, [0, 180.5, 180.5, 0])
        pdb.gimp_eraser_default(stripesPat, 4, [0, 194, 194, 0])
        pdb.gimp_eraser_default(stripesPat, 4, [56.5, 250, 250, 56.5])
        pdb.gimp_eraser_default(stripesPat, 4, [69, 250, 250, 69])
        pdb.gimp_eraser_default(stripesPat, 4, [180.5, 250, 250, 180.5])
        pdb.gimp_eraser_default(stripesPat, 4, [194, 250, 250, 194])
        
        #Copy the pattern to clipboard and delete it
        non_empty = pdb.gimp_edit_copy(stripesPat)
        pdb.gimp_image_remove_layer(img, stripesPat)
    
    #Create the candy stripes effect
    stripesLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Stripes", 100, LAYER_MODE_DARKEN_ONLY)
    pdb.gimp_image_insert_layer(img, stripesLayer, None, 1)
    pdb.gimp_drawable_fill(stripesLayer, FILL_TRANSPARENT)
    pdb.gimp_image_select_item(img, 2, textLayer)
    pdb.gimp_context_set_pattern("Clipboard Image")
    pdb.gimp_bucket_fill(stripesLayer, BUCKET_FILL_PATTERN, LAYER_MODE_NORMAL, 100, 255,  FALSE, 0, 0)
    pdb.gimp_selection_none(img)
    
    #Resets the previous user defaults
    pdb.gimp_context_pop() 
    
def pm_candy_cane_text_atl(img, layer, offset_x, offset_y, blurRadius, opacity, stripcolor_1, stripcolor_2, chrCane):
    
    #Initiates the temporary state
    pdb.gimp_context_push()
		
    #Make the background active
    img.layers[-1]
    backLayer = img.layers[-1]
    pdb.gimp_image_set_active_layer(img, backLayer)
    pdb.gimp_item_set_name(backLayer, "Background")
    pdb.gimp_context_set_background("#73bde0")
    pdb.gimp_drawable_fill(backLayer, FILL_BACKGROUND)
    
    #Make the text active
    img.layers[0]
    textLayer = img.layers[0]
    textLayer = pdb.gimp_image_set_active_layer(img, textLayer)      
    if textLayer==False:
        pdb.gimp_message_set_handler(0)
        pdb.gimp_message("Need to create a Text")       
        gimp.quit()
        
	#Make active the text 
    textLayer = pdb.gimp_image_get_active_drawable(img)
    pdb.gimp_image_set_active_layer(img, textLayer)
    pdb.gimp_item_set_name(textLayer, "Text")
    text_layer = pdb.gimp_item_is_text_layer(textLayer)
    line_spacing = pdb.gimp_text_layer_get_line_spacing(textLayer)
    pdb.gimp_text_layer_set_line_spacing(textLayer, -40)
    pdb.gimp_text_layer_set_letter_spacing(textLayer, 10)
    pdb.gimp_image_select_item(img, 2, textLayer)
    pdb.plug_in_antialias(img, textLayer)
    pdb.gimp_context_set_foreground("#b1b1b1")
    pdb.gimp_bucket_fill(textLayer, BUCKET_FILL_FG, LAYER_MODE_NORMAL, 100, 255,  FALSE, 0, 0)
    pdb.gimp_selection_none(img)
    wh = pdb.gimp_drawable_width(textLayer)
    hh = pdb.gimp_drawable_height(textLayer)
    w = pdb.gimp_image_width(img)
    h = pdb.gimp_image_height(img)    
    factor = min (float(img.width) / textLayer.width, float(img.height) / textLayer.height)
    textLayer.scale(int(textLayer.width * factor-150), int(textLayer.height * factor-50))
    x = textLayer.offsets[0]
    y = textLayer.offsets[1]
    x = (img.width - textLayer.width) / 2
    y = (img.height - textLayer.height) / 2
    textLayer.set_offsets(x, y)
    pdb.gimp_layer_resize_to_image_size(textLayer)
    
    pdb.script_fu_drop_shadow(img, textLayer, offset_x, offset_y, blurRadius, "Black", opacity, FALSE)
    
    #Bump the text
    pdb.gimp_context_set_default_colors()
    bumpLayer = pdb.gimp_layer_new_from_drawable(textLayer, img)
    pdb.gimp_image_insert_layer(img, bumpLayer, None, 0)
    pdb.gimp_item_set_name(bumpLayer, "Bumpmap")
    pdb.gimp_layer_set_mode(bumpLayer, LAYER_MODE_NORMAL)
    pdb.gimp_layer_set_opacity(bumpLayer, 100)
    pdb.gimp_layer_flatten(bumpLayer)
    pdb.plug_in_gauss(img, bumpLayer, 40, 40, 1)   #method=1
    pdb.gimp_layer_set_opacity(bumpLayer, 0)
    pdb.plug_in_bump_map(img, textLayer, 
                                bumpLayer,   #bumpmap
                                      270,   #azimuth
                                       45,   #elevation
                                       55,   #depth
                                        0,   #xofs
                                        8,   #yofs
                                        0,   #waterlevel
                                        0,   #ambient
                                     TRUE,   #compensate
                                    FALSE,   #invert
                                        0)   #type - SPHERICAL
    
    #Smooth the text
    pdb.gimp_layer_set_lock_alpha(textLayer, TRUE)
    pdb.plug_in_gauss(img, textLayer, 5, 5, 1)   #method=1
    pdb.gimp_layer_set_lock_alpha(textLayer, FALSE)
    
    #Create the candy stripes pattern
    size = 250
    stripesPat = pdb.gimp_layer_new(img, size, size, RGBA_IMAGE, "Candy Stripes", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_insert_layer(img, stripesPat, None, 1)
    pdb.gimp_drawable_fill(stripesPat, FILL_WHITE)
    pdb.gimp_brushes_set_brush("2. Block 01")
    pdb.gimp_context_set_brush_size(440)
    pdb.gimp_context_set_brush_aspect_ratio(17)
    pdb.gimp_context_set_brush_angle(-45)
    pdb.gimp_context_set_brush_default_hardness()
    pdb.gimp_context_set_brush_spacing(0.01)
    pdb.gimp_context_set_brush_force(1)
    #Color 1
    pdb.gimp_context_set_foreground(stripcolor_1)
    pdb.gimp_context_set_dynamics("Dynamics Off")  
    pdb.gimp_paintbrush_default(stripesPat, 2, [0, 0])
    pdb.gimp_paintbrush_default(stripesPat, 2, [65, 60])
    pdb.gimp_paintbrush_default(stripesPat, 2, [125, 125])
    pdb.gimp_paintbrush_default(stripesPat, 2, [187, 188])
    pdb.gimp_paintbrush_default(stripesPat, 2, [250, 250])
    #Color 2
    pdb.gimp_context_set_foreground(stripcolor_2)
    pdb.gimp_context_set_dynamics("Dynamics Off")  
    pdb.gimp_paintbrush_default(stripesPat, 2, [32, 30])
    pdb.gimp_paintbrush_default(stripesPat, 2, [127, 60])
    pdb.gimp_paintbrush_default(stripesPat, 2, [187, 125])
    pdb.gimp_paintbrush_default(stripesPat, 2, [219, 219])
    
    #Copy the pattern to clipboard and delete it
    non_empty = pdb.gimp_edit_copy(stripesPat)
    pdb.gimp_image_remove_layer(img, stripesPat)
    
    if chrCane==1:
        #Create the candy stripes pattern
        size = 250
        stripesPat = pdb.gimp_layer_new(img, size, size, RGB_IMAGE, "Candy Stripes", 100, LAYER_MODE_NORMAL)
        pdb.gimp_image_insert_layer(img, stripesPat, None, 1)
        pdb.gimp_drawable_fill(stripesPat, FILL_WHITE)
        pdb.gimp_brushes_set_brush("2. Block 01")
        pdb.gimp_context_set_brush_size(440)
        pdb.gimp_context_set_brush_aspect_ratio(17)
        pdb.gimp_context_set_brush_angle(-45)
        pdb.gimp_context_set_brush_default_hardness()
        pdb.gimp_context_set_brush_spacing(0.01)
        pdb.gimp_context_set_brush_force(1)
        #Color 1
        pdb.gimp_context_set_foreground(stripcolor_1)
        pdb.gimp_context_set_dynamics("Dynamics Off")  
        pdb.gimp_paintbrush_default(stripesPat, 2, [0, 0])
        pdb.gimp_paintbrush_default(stripesPat, 2, [65, 60])
        pdb.gimp_paintbrush_default(stripesPat, 2, [125, 125])
        pdb.gimp_paintbrush_default(stripesPat, 2, [187, 188])
        pdb.gimp_paintbrush_default(stripesPat, 2, [250, 250])
        #Color 2
        pdb.gimp_context_set_brush_size(440)
        pdb.gimp_context_set_brush_aspect_ratio(20)
        pdb.gimp_context_set_foreground(stripcolor_2)
        pdb.gimp_context_set_dynamics("Dynamics Off")  
        pdb.gimp_paintbrush_default(stripesPat, 2, [32, 30])
        pdb.gimp_paintbrush_default(stripesPat, 2, [127, 60])
        pdb.gimp_paintbrush_default(stripesPat, 2, [187, 125])
        pdb.gimp_paintbrush_default(stripesPat, 2, [219, 219])
        
        pdb.gimp_brushes_set_brush("2. Block 01")
        pdb.gimp_context_set_brush_size(5)
        pdb.gimp_context_set_brush_aspect_ratio(0)
        pdb.gimp_context_set_brush_angle(0)
        pdb.gimp_context_set_brush_default_hardness()
        pdb.gimp_context_set_brush_spacing(0.01)
        pdb.gimp_context_set_brush_force(1)
        pdb.gimp_context_set_dynamics("Dynamics Off")  
        pdb.gimp_eraser_default(stripesPat, 4, [0, 56.5, 56.5, 0])
        pdb.gimp_eraser_default(stripesPat, 4, [0, 69, 69, 0])
        pdb.gimp_eraser_default(stripesPat, 4, [0, 180.5, 180.5, 0])
        pdb.gimp_eraser_default(stripesPat, 4, [0, 194, 194, 0])
        pdb.gimp_eraser_default(stripesPat, 4, [56.5, 250, 250, 56.5])
        pdb.gimp_eraser_default(stripesPat, 4, [69, 250, 250, 69])
        pdb.gimp_eraser_default(stripesPat, 4, [180.5, 250, 250, 180.5])
        pdb.gimp_eraser_default(stripesPat, 4, [194, 250, 250, 194])
        
        #Copy the pattern to clipboard and delete it
        non_empty = pdb.gimp_edit_copy(stripesPat)
        pdb.gimp_image_remove_layer(img, stripesPat)
    
    #Create the candy stripes effect
    stripesLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Stripes", 100, LAYER_MODE_DARKEN_ONLY)
    pdb.gimp_image_insert_layer(img, stripesLayer, None, 1)
    pdb.gimp_drawable_fill(stripesLayer, FILL_TRANSPARENT)
    pdb.gimp_image_select_item(img, 2, textLayer)
    pdb.gimp_context_set_pattern("Clipboard Image")
    pdb.gimp_bucket_fill(stripesLayer, BUCKET_FILL_PATTERN, LAYER_MODE_NORMAL, 100, 255,  FALSE, 0, 0)
    pdb.gimp_selection_none(img)
        
    #Resets the previous user defaults
    pdb.gimp_context_pop()  
    
register(
    "pm_candy_cane_text_atl",
    "Creates a candy text",
    "Creates a candy text",
    "Pocholo",
    "Pocholo",
    "2022",
    "Candy Cane",
    "",
    [
    (PF_IMAGE,    "img", "Input image", None),    #current image
    (PF_DRAWABLE, "drawable", "Input layer", None), #current layer
    (PF_SPINNER,  "offset_x", "Offset X:", 0, (-100, 100, 1)),
    (PF_SPINNER,  "offset_y", "Offset Y:", 10, (-100, 100, 1)),   
	(PF_SPINNER,  "blurRadius", "Blur Radius:", 10, (0, 100, 1)),
    (PF_SPINNER,  "opacity", "Shadow Opacity:", 100, (1, 100, 1)),
    (PF_COLOR,    "stripcolor_1", "Stripes Color 1", (255, 35, 15)),
    (PF_COLOR,    "stripcolor_2", "Stripes Color 2", (30, 200, 138)),
    (PF_BOOL,     "chrCane", "Christmas Candy Cane?:", 0),
    ],
    [],
    pm_candy_cane_text_atl, menu="<Image>/Filters/Alpha to Logo",
    domain=("gimp20", gimp.locale_directory))

register(
    "pm_candy_cane_text",
    "Creates a candy text",
    "Creates a candy text",
    "Pocholo",
    "Pocholo",
    "2022",
    "Candy Cane",
    "",
    [
    (PF_TEXT, "customtext", "Text string", "Candy\nCane"),
    (PF_FONT, "fontname", "Font name", "Arial Rounded MT Bold,"),
    (PF_SPINNER, "offset_x", "Offset X:", 0, (-100, 100, 1)),
    (PF_SPINNER, "offset_y", "Offset Y:", 10, (-100, 100, 1)),   
	(PF_SPINNER, "blurRadius", "Blur Radius:", 10, (0, 100, 1)),
    (PF_SPINNER, "opacity", "Shadow Opacity:", 100, (1, 100, 1)),
    (PF_COLOR, "stripcolor_1", "Stripes Color 1", (255, 35, 15)),
    (PF_COLOR, "stripcolor_2", "Stripes Color 2", (30, 200, 138)),
    (PF_BOOL,     "chrCane", "Christmas Candy Cane?:", 0),
    ],
    [],
    pm_candy_cane_text, menu="<Image>/Python-Fu",
    domain=("gimp20", gimp.locale_directory))
main()